@@ -1,12 +1,16 @@
 package sctp
 
 import (
+	"context"
+	"encoding/json"
+	"errors"
 	"fmt"
 	"io/ioutil"
 	"os"
 	"strings"
 	"time"
 
+	igntypes "github.com/coreos/ignition/config/v2_2/types"
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/ginkgo/extensions/table"
 	. "github.com/onsi/gomega"
@@ -18,9 +22,11 @@ import (
 	"k8s.io/apimachinery/pkg/util/intstr"
 
 	"github.com/openshift-kni/cnf-features-deploy/functests/utils/client"
+	"github.com/openshift-kni/cnf-features-deploy/functests/utils/discovery"
 	"github.com/openshift-kni/cnf-features-deploy/functests/utils/execute"
 	"github.com/openshift-kni/cnf-features-deploy/functests/utils/images"
 	"github.com/openshift-kni/cnf-features-deploy/functests/utils/namespaces"
+	utilNodes "github.com/openshift-kni/cnf-features-deploy/functests/utils/nodes"
 
 	"k8s.io/utils/pointer"
 )
@@ -30,6 +36,11 @@ const hostnameLabel = "kubernetes.io/hostname"
 const TestNamespace = "sctptest"
 const defaultNamespace = "default"
 
+const (
+	sctpBlacklistPath = "/etc/modprobe.d/sctp-blacklist.conf"
+	sctpLoadPath      = "/etc/modules-load.d/sctp-load.conf"
+)
+
 var (
 	sctpNodeSelector string
 	hasNonCnfWorkers bool
@@ -43,12 +54,10 @@ type nodesInfo struct {
 
 func init() {
 	roleWorkerCNF := os.Getenv("ROLE_WORKER_CNF")
-	if roleWorkerCNF == "" {
-		roleWorkerCNF = "worker-cnf"
+	if roleWorkerCNF != "" {
+		sctpNodeSelector = fmt.Sprintf("node-role.kubernetes.io/%s=", roleWorkerCNF)
 	}
 
-	sctpNodeSelector = fmt.Sprintf("node-role.kubernetes.io/%s=", roleWorkerCNF)
-
 	hasNonCnfWorkers = true
 	if os.Getenv("SCTPTEST_HAS_NON_CNF_WORKERS") == "false" {
 		hasNonCnfWorkers = false
@@ -64,12 +73,18 @@ var _ = Describe("sctp", func() {
 		// error looking up service account sctptest/default: serviceaccount \"default\" not found"
 		// Making sure the account is there, and scream if it's not being created after 5 minutes
 		Eventually(func() error {
-			_, err := client.Client.ServiceAccounts(TestNamespace).Get("default", metav1.GetOptions{})
+			_, err := client.Client.ServiceAccounts(TestNamespace).Get(context.Background(), "default", metav1.GetOptions{})
 			return err
 		}, 5*time.Minute, 5*time.Second).Should(Not(HaveOccurred()))
 
 		err = namespaces.Clean(TestNamespace, "testsctp-", client.Client)
 		Expect(err).ToNot(HaveOccurred())
+
+		if sctpNodeSelector == "" {
+			sctpNodeSelector, err = findSCTPNodeSelector()
+			Expect(err).ToNot(HaveOccurred())
+		}
+
 	})
 
 	var _ = Describe("Negative - Sctp disabled", func() {
@@ -85,12 +100,20 @@ var _ = Describe("sctp", func() {
 
 			namespaces.Clean(TestNamespace, "testsctp-", client.Client)
 			By("Choosing the nodes for the server and the client")
-			nodes, err := client.Client.Nodes().List(metav1.ListOptions{
+			nodes, err := client.Client.Nodes().List(context.Background(), metav1.ListOptions{
 				LabelSelector: "node-role.kubernetes.io/worker,!" + strings.Replace(sctpNodeSelector, "=", "", -1),
 			})
 			Expect(err).ToNot(HaveOccurred())
-			Expect(len(nodes.Items)).To(BeNumerically(">", 0))
-			serverNode = nodes.Items[0].ObjectMeta.Labels[hostnameLabel]
+
+			filtered, err := utilNodes.MatchingOptionalSelector(nodes.Items)
+			Expect(err).ToNot(HaveOccurred())
+
+			if discovery.Enabled() && len(filtered) == 0 {
+				Skip("Did not find a node without sctp module enabled")
+			} else {
+				Expect(len(filtered)).To(BeNumerically(">", 0))
+			}
+			serverNode = filtered[0].ObjectMeta.Labels[hostnameLabel]
 
 			createSctpService(client.Client, TestNamespace)
 		})
@@ -107,11 +130,11 @@ var _ = Describe("sctp", func() {
 						ContainerPort: 30101,
 					},
 				}
-				serverPod, err := client.Client.Pods(TestNamespace).Create(pod)
+				serverPod, err := client.Client.Pods(TestNamespace).Create(context.Background(), pod, metav1.CreateOptions{})
 				Expect(err).ToNot(HaveOccurred())
 				By("Checking the server pod fails")
 				Eventually(func() k8sv1.PodPhase {
-					runningPod, err := client.Client.Pods(TestNamespace).Get(serverPod.Name, metav1.GetOptions{})
+					runningPod, err := client.Client.Pods(TestNamespace).Get(context.Background(), serverPod.Name, metav1.GetOptions{})
 					Expect(err).ToNot(HaveOccurred())
 					return runningPod.Status.Phase
 				}, 1*time.Minute, 1*time.Second).Should(Equal(k8sv1.PodFailed))
@@ -123,7 +146,7 @@ var _ = Describe("sctp", func() {
 
 		execute.BeforeAll(func() {
 			checkForSctpReady(client.Client)
-			nodes = selectSctpNodes()
+			nodes = selectSctpNodes(sctpNodeSelector)
 		})
 
 		Context("Connectivity between client and server", func() {
@@ -167,7 +190,7 @@ var _ = Describe("sctp", func() {
 			)
 			// OCP-26763
 			It("Should connect a client pod to a server pod. Feature LatencySensitive Active", func() {
-				fg, err := client.Client.FeatureGates().Get("cluster", metav1.GetOptions{})
+				fg, err := client.Client.FeatureGates().Get(context.Background(), "cluster", metav1.GetOptions{})
 				Expect(err).ToNot(HaveOccurred())
 				if fg.Spec.FeatureSet == "LatencySensitive" {
 					By("Starting the server")
@@ -226,24 +249,36 @@ func loadMC() *mcfgv1.MachineConfig {
 	return mc
 }
 
-func selectSctpNodes() nodesInfo {
-	By("Choosing the nodes for the server and the client")
-	nodes, err := client.Client.Nodes().List(metav1.ListOptions{
-		LabelSelector: sctpNodeSelector,
+func getSCTPNodes(selector string) []k8sv1.Node {
+	nodes, err := client.Client.Nodes().List(context.Background(), metav1.ListOptions{
+		LabelSelector: selector,
 	})
 	Expect(err).ToNot(HaveOccurred())
-	Expect(len(nodes.Items)).To(BeNumerically(">", 0))
+	filtered, err := utilNodes.MatchingOptionalSelector(nodes.Items)
+	Expect(err).ToNot(HaveOccurred())
 
-	client := nodes.Items[0].ObjectMeta.Labels[hostnameLabel]
-	server := nodes.Items[0].ObjectMeta.Labels[hostnameLabel]
-	clientNodeIP := nodes.Items[0].Status.Addresses[0].Address
-	if len(nodes.Items) > 1 {
-		server = nodes.Items[1].ObjectMeta.Labels[hostnameLabel]
-	}
-	return nodesInfo{clientNode: client, serverNode: server, nodeAddress: clientNodeIP}
+	Expect(len(filtered)).To(BeNumerically(">", 0))
+	return filtered
 }
 
-func startServerPod(node, namespace string) *k8sv1.Pod {
+func nodesToInfo(client, server k8sv1.Node) nodesInfo {
+	clientHost := client.ObjectMeta.Labels[hostnameLabel]
+	serverHost := server.ObjectMeta.Labels[hostnameLabel]
+	clientNodeIP := client.Status.Addresses[0].Address
+	return nodesInfo{clientNode: clientHost, serverNode: serverHost, nodeAddress: clientNodeIP}
+}
+
+func selectSctpNodes(selector string) nodesInfo {
+	By("Choosing the nodes for the server and the client")
+
+	filtered := getSCTPNodes(selector)
+	if len(filtered) > 1 {
+		return nodesToInfo(filtered[0], filtered[1])
+	}
+	return nodesToInfo(filtered[0], filtered[0])
+}
+
+func startServerPod(node, namespace string, networks ...string) *k8sv1.Pod {
 	serverArgs := []string{"-ip", "0.0.0.0", "-port", "30101", "-server"}
 	pod := sctpTestPod("testsctp-server", node, "sctpserver", namespace, serverArgs)
 	pod.Spec.Containers[0].Ports = []k8sv1.ContainerPort{
@@ -253,12 +288,18 @@ func startServerPod(node, namespace string) *k8sv1.Pod {
 			ContainerPort: 30101,
 		},
 	}
-	serverPod, err := client.Client.Pods(namespace).Create(pod)
+
+	if networks != nil {
+		pod.Annotations = map[string]string{"k8s.v1.cni.cncf.io/networks": strings.Join(networks, ",")}
+	}
+
+	serverPod, err := client.Client.Pods(namespace).Create(context.Background(), pod, metav1.CreateOptions{})
 	Expect(err).ToNot(HaveOccurred())
+
 	var res *k8sv1.Pod
 	By("Fetching the server's ip address")
 	Eventually(func() k8sv1.PodPhase {
-		res, err = client.Client.Pods(namespace).Get(serverPod.Name, metav1.GetOptions{})
+		res, err = client.Client.Pods(namespace).Get(context.Background(), serverPod.Name, metav1.GetOptions{})
 		Expect(err).ToNot(HaveOccurred())
 		return res.Status.Phase
 	}, 3*time.Minute, 1*time.Second).Should(Equal(k8sv1.PodRunning))
@@ -266,20 +307,23 @@ func startServerPod(node, namespace string) *k8sv1.Pod {
 }
 
 func checkForSctpReady(cs *client.ClientSet) {
-	nodes, err := cs.Nodes().List(metav1.ListOptions{
+	nodes, err := cs.Nodes().List(context.Background(), metav1.ListOptions{
 		LabelSelector: sctpNodeSelector,
 	})
 	Expect(err).ToNot(HaveOccurred())
-	Expect(len(nodes.Items)).To(BeNumerically(">", 0))
+
+	filtered, err := utilNodes.MatchingOptionalSelector(nodes.Items)
+	Expect(err).ToNot(HaveOccurred())
+	Expect(len(filtered)).To(BeNumerically(">", 0))
 
 	args := []string{`set -x; x="$(checksctp 2>&1)"; echo "$x" ; if [ "$x" = "SCTP supported" ]; then echo "succeeded"; exit 0; else echo "failed"; exit 1; fi`}
-	for _, n := range nodes.Items {
+	for _, n := range filtered {
 		job := jobForNode("testsctp-check", n.ObjectMeta.Labels[hostnameLabel], "checksctp", []string{"/bin/bash", "-c"}, args)
-		cs.Pods(TestNamespace).Create(job)
+		cs.Pods(TestNamespace).Create(context.Background(), job, metav1.CreateOptions{})
 	}
 
 	Eventually(func() bool {
-		pods, err := cs.Pods(TestNamespace).List(metav1.ListOptions{LabelSelector: "app=checksctp"})
+		pods, err := cs.Pods(TestNamespace).List(context.Background(), metav1.ListOptions{LabelSelector: "app=checksctp"})
 		ExpectWithOffset(1, err).ToNot(HaveOccurred())
 
 		for _, p := range pods.Items {
@@ -291,16 +335,21 @@ func checkForSctpReady(cs *client.ClientSet) {
 	}, 10*time.Minute, 10*time.Second).Should(Equal(true))
 }
 
-func testClientServerConnection(cs *client.ClientSet, namespace string, destIP string, port int32, clientNode string, serverPodName string, shouldSucceed bool) {
+func testClientServerConnection(cs *client.ClientSet, namespace string, destIP string, port int32, clientNode string, serverPodName string, shouldSucceed bool, networks ...string) {
 	By("Connecting a client to the server")
 	clientArgs := []string{"-ip", destIP, "-port",
 		fmt.Sprint(port), "-lport", "30102"}
 	clientPod := sctpTestPod("testsctp-client", clientNode, "sctpclient", namespace, clientArgs)
-	cs.Pods(namespace).Create(clientPod)
+	if networks != nil {
+		clientPod.Annotations = map[string]string{"k8s.v1.cni.cncf.io/networks": strings.Join(networks, ",")}
+	}
+
+	_, err := cs.Pods(namespace).Create(context.Background(), clientPod, metav1.CreateOptions{})
+	Expect(err).ToNot(HaveOccurred())
 
 	if !shouldSucceed {
 		Consistently(func() k8sv1.PodPhase {
-			pod, err := cs.Pods(namespace).Get(serverPodName, metav1.GetOptions{})
+			pod, err := cs.Pods(namespace).Get(context.Background(), serverPodName, metav1.GetOptions{})
 			Expect(err).ToNot(HaveOccurred())
 			return pod.Status.Phase
 		}, 30*time.Second, 1*time.Second).Should(Equal(k8sv1.PodRunning))
@@ -308,7 +357,7 @@ func testClientServerConnection(cs *client.ClientSet, namespace string, destIP s
 	}
 
 	Eventually(func() k8sv1.PodPhase {
-		pod, err := cs.Pods(namespace).Get(serverPodName, metav1.GetOptions{})
+		pod, err := cs.Pods(namespace).Get(context.Background(), serverPodName, metav1.GetOptions{})
 		Expect(err).ToNot(HaveOccurred())
 		return pod.Status.Phase
 	}, 1*time.Minute, 1*time.Second).Should(Equal(k8sv1.PodSucceeded))
@@ -338,7 +387,7 @@ func createSctpService(cs *client.ClientSet, namespace string) *k8sv1.Service {
 			Type: "NodePort",
 		},
 	}
-	activeService, err := cs.Services(namespace).Create(&service)
+	activeService, err := cs.Services(namespace).Create(context.Background(), &service, metav1.CreateOptions{})
 	Expect(err).ToNot(HaveOccurred())
 	return activeService
 }
@@ -436,7 +485,7 @@ func setupIngress(namespace, fromPod, toPod string, port int32) error {
 			},
 		},
 	}
-	_, err := client.Client.NetworkPolicies(namespace).Create(policy)
+	_, err := client.Client.NetworkPolicies(namespace).Create(context.Background(), policy, metav1.CreateOptions{})
 	return err
 }
 
@@ -477,6 +526,79 @@ func setupEgress(namespace, fromPod, toPod string, port int32) error {
 			},
 		},
 	}
-	_, err := client.Client.NetworkPolicies(namespace).Create(policy)
+	_, err := client.Client.NetworkPolicies(namespace).Create(context.Background(), policy, metav1.CreateOptions{})
 	return err
 }
+
+func findSCTPNodeSelector() (string, error) {
+	mcList, err := client.Client.MachineConfigs().List(context.Background(), metav1.ListOptions{})
+	Expect(err).ToNot(HaveOccurred())
+
+	for _, mc := range mcList.Items {
+		enables, err := enablesSCTP(mc)
+		if err != nil {
+			return "", err
+		}
+		if enables {
+			mcLabel, found := mc.ObjectMeta.Labels["machineconfiguration.openshift.io/role"]
+			if !found {
+				continue
+			}
+
+			sctpNodeSelector, err := findSCTPNodeSelectorByMCLabel(mcLabel)
+			if err != nil {
+				continue
+			}
+
+			return sctpNodeSelector, nil
+		}
+	}
+
+	return "", errors.New("Cannot find a machine configuration that enables SCTP")
+}
+
+func enablesSCTP(mc mcfgv1.MachineConfig) (bool, error) {
+	blacklistPathFound := false
+	loadPathFound := false
+
+	ignitionConfig := igntypes.Config{}
+	if mc.Spec.Config.Raw == nil {
+		return false, nil
+	}
+
+	err := json.Unmarshal(mc.Spec.Config.Raw, &ignitionConfig)
+	if err != nil {
+		return false, fmt.Errorf("Failed to unmarshal ignition config %v", err)
+	}
+
+	for _, file := range ignitionConfig.Storage.Files {
+		if !blacklistPathFound && file.Path == sctpBlacklistPath {
+			blacklistPathFound = true
+		}
+		if !loadPathFound && file.Path == sctpLoadPath {
+			loadPathFound = true
+		}
+	}
+
+	return (blacklistPathFound && loadPathFound), nil
+}
+
+func findSCTPNodeSelectorByMCLabel(mcLabel string) (string, error) {
+	mcpList, err := client.Client.MachineConfigPools().List(context.Background(), metav1.ListOptions{})
+	Expect(err).ToNot(HaveOccurred())
+
+	for _, mcp := range mcpList.Items {
+		for _, lsr := range mcp.Spec.MachineConfigSelector.MatchExpressions {
+			for _, value := range lsr.Values {
+				if value == mcLabel {
+					for key, label := range mcp.Spec.NodeSelector.MatchLabels {
+						newSCTPNodeSelector := key + "=" + label
+						return newSCTPNodeSelector, nil
+					}
+				}
+			}
+		}
+	}
+
+	return "", errors.New("Cannot find SCTPNodeSelector")
+}
